@@ -21,9 +21,10 @@ Example:
 """
 from __future__ import annotations
 
-from typing import Any, List, Optional
+from typing import Any, List, Optional, Union
 
-from rdflib import Graph, Namespace, RDF, URIRef
+from rdflib import Graph, Literal, Namespace, RDF, URIRef
+from skolemizer import Skolemizer
 
 from .catalogrecord import CatalogRecord
 from .dataservice import DataService
@@ -36,6 +37,7 @@ DCT = Namespace("http://purl.org/dc/terms/")
 DCAT = Namespace("http://www.w3.org/ns/dcat#")
 FOAF = Namespace("http://xmlns.com/foaf/0.1/")
 MODELLDCATNO = Namespace("https://data.norge.no/vocabulary/modelldcatno#")
+DCATNO = Namespace("https://data.norge.no/vocabulary/dcatno#")
 
 
 class Catalog(Dataset):
@@ -57,6 +59,7 @@ class Catalog(Dataset):
                     the registration of a single dataset or data service \
                     that is part of the catalog.
         models (List[URI]): A list of links to InformationModels
+        contains_services (List[URI]): A list of links to Services
     """
 
     __slots__ = (
@@ -68,6 +71,8 @@ class Catalog(Dataset):
         "_catalogs",
         "_catalogrecords",
         "_models",
+        "_contains_services",
+        "_dct_identifier",
     )
 
     _homepage: URI
@@ -78,10 +83,16 @@ class Catalog(Dataset):
     _catalogs: List[Catalog]
     _catalogrecords: List[CatalogRecord]
     _models: List[Any]
+    _contains_services: List[Any]
+    _dct_identifier: str
 
-    def __init__(self) -> None:
+    def __init__(self, identifier: Optional[str] = None) -> None:
         """Inits catalog object with default values."""
         super().__init__()
+
+        if identifier:
+            self.identifier = identifier
+
         self._type = DCAT.Catalog
         self.themes = []
         self.has_parts = []
@@ -90,6 +101,7 @@ class Catalog(Dataset):
         self.catalogs = []
         self.catalogrecords = []
         self.models = []
+        self.contains_services = []
 
     @property
     def homepage(self: Catalog) -> str:
@@ -136,6 +148,15 @@ class Catalog(Dataset):
     def models(self: Catalog, models: List[Any]) -> None:
         self._models = models
 
+    @property
+    def contains_services(self: Catalog) -> List[Any]:
+        """Get/set for contains_services."""
+        return self._contains_services
+
+    @contains_services.setter
+    def contains_services(self: Catalog, contains_services: List[Any]) -> None:
+        self._contains_services = contains_services
+
     @property
     def services(self: Catalog) -> List[DataService]:
         """Get/set for services."""
@@ -163,6 +184,16 @@ class Catalog(Dataset):
     def catalogrecords(self: Catalog, catalogrecords: List[CatalogRecord]) -> None:
         self._catalogrecords = catalogrecords
 
+    @property
+    def dct_identifier(self) -> str:
+        """Get for dct_identifier."""
+        return self._dct_identifier
+
+    @dct_identifier.setter
+    def dct_identifier(self, dct_identifier: str) -> None:
+        """Set for dct_identifier."""
+        self._dct_identifier = dct_identifier
+
         # -
 
     def to_rdf(
@@ -172,7 +203,8 @@ class Catalog(Dataset):
         include_datasets: bool = True,
         include_services: bool = True,
         include_models: bool = True,
-    ) -> bytes:
+        include_contains_services: bool = True,
+    ) -> Union[bytes, str]:
         """Maps the catalog to rdf.
 
         Available formats:
@@ -186,12 +218,16 @@ class Catalog(Dataset):
             include_datasets (bool): includes the dataset graphs in the catalog
             include_services (bool): includes the services in the catalog
             include_models (bool): includes the models in the catalog
+            include_contains_services (bool): includes the services (cpsvno) in the catalog
 
         Returns:
             a rdf serialization as a bytes literal according to format.
         """
         return self._to_graph(
-            include_datasets, include_services, include_models
+            include_datasets,
+            include_services,
+            include_models,
+            include_contains_services,
         ).serialize(format=format, encoding=encoding)
 
     # -
@@ -201,11 +237,19 @@ class Catalog(Dataset):
         include_datasets: bool = True,
         include_services: bool = True,
         include_models: bool = True,
+        include_contains_services: bool = True,
     ) -> Graph:
+
+        if not getattr(self, "identifier", None):
+            self.identifier = Skolemizer.add_skolemization()
+
         super(Catalog, self)._to_graph()
+        self._g.bind("modelldcatno", MODELLDCATNO)
+        self._g.bind("dcatno", DCATNO)
 
         self._g.add((URIRef(self.identifier), RDF.type, self._type))
 
+        self._dct_identifier_to_graph()
         self._homepage_to_graph()
         self._themes_to_graph()
         self._has_parts_to_graph()
@@ -214,6 +258,7 @@ class Catalog(Dataset):
         self._catalogs_to_graph()
         self._catalogrecords_to_graph()
         self._models_to_graph()
+        self._contains_services_to_graph()
 
         # Add all the datasets to the graf
         if include_datasets:
@@ -230,8 +275,23 @@ class Catalog(Dataset):
             for model in self._models:
                 self._g += model._to_graph()
 
+        # Add all the contains_services  to the graf
+        if include_contains_services:
+            for service in self._contains_services:
+                self._g += service._to_graph()
+
         return self._g
 
+    def _dct_identifier_to_graph(self: Dataset) -> None:
+        if getattr(self, "dct_identifier", None):
+            self._g.add(
+                (
+                    URIRef(self.identifier),
+                    DCT.identifier,
+                    Literal(self.dct_identifier),
+                )
+            )
+
     def _homepage_to_graph(self: Catalog) -> None:
         if getattr(self, "homepage", None):
             self._g.add((URIRef(self.identifier), FOAF.homepage, URIRef(self.homepage)))
@@ -248,29 +308,58 @@ class Catalog(Dataset):
                 )
 
     def _has_parts_to_graph(self: Catalog) -> None:
+
         if getattr(self, "has_parts", None):
-            for _resource in self._has_parts:
+
+            for has_parts in self._has_parts:
+
+                if not getattr(has_parts, "identifier", None):
+                    has_parts.identifier = Skolemizer.add_skolemization()
+
                 self._g.add(
-                    (URIRef(self.identifier), DCT.hasPart, URIRef(_resource.identifier))
+                    (URIRef(self.identifier), DCT.hasPart, URIRef(has_parts.identifier))
                 )
 
     def _datasets_to_graph(self: Catalog) -> None:
         if getattr(self, "datasets", None):
             for _dataset in self._datasets:
+
+                if not getattr(_dataset, "identifier", None):
+                    _dataset.identifier = Skolemizer.add_skolemization()
+
                 self._g.add(
-                    (URIRef(self.identifier), DCAT.dataset, URIRef(_dataset.identifier))
+                    (
+                        URIRef(self.identifier),
+                        DCAT.dataset,
+                        URIRef(_dataset.identifier),
+                    )
                 )
 
     def _services_to_graph(self: Catalog) -> None:
+
         if getattr(self, "services", None):
+
             for _service in self._services:
+
+                if not getattr(_service, "identifier", None):
+                    _service.identifier = Skolemizer.add_skolemization()
+
                 self._g.add(
-                    (URIRef(self.identifier), DCAT.service, URIRef(_service.identifier))
+                    (
+                        URIRef(self.identifier),
+                        DCAT.service,
+                        URIRef(_service.identifier),
+                    )
                 )
 
     def _catalogs_to_graph(self: Catalog) -> None:
         if getattr(self, "catalogs", None):
+
             for _catalog in self._catalogs:
+
+                if not getattr(_catalog, "identifier", None):
+                    _catalog.identifier = Skolemizer.add_skolemization()
+
                 self._g.add(
                     (URIRef(self.identifier), DCAT.catalog, URIRef(_catalog.identifier))
                 )
@@ -278,6 +367,10 @@ class Catalog(Dataset):
     def _catalogrecords_to_graph(self: Catalog) -> None:
         if getattr(self, "catalogrecords", None):
             for _catalogrecord in self._catalogrecords:
+
+                if not getattr(_catalogrecord, "identifier", None):
+                    _catalogrecord.identifier = Skolemizer.add_skolemization()
+
                 self._g.add(
                     (
                         URIRef(self.identifier),
@@ -296,3 +389,14 @@ class Catalog(Dataset):
                         URIRef(_model.identifier),
                     )
                 )
+
+    def _contains_services_to_graph(self: Catalog) -> None:
+        if getattr(self, "contains_services", None):
+            for _service in self._contains_services:
+                self._g.add(
+                    (
+                        URIRef(self.identifier),
+                        DCATNO.containsService,
+                        URIRef(_service.identifier),
+                    )
+                )
