@@ -16,9 +16,10 @@ Example:
 """
 from __future__ import annotations
 
-from typing import Optional, TYPE_CHECKING
+from typing import Optional, TYPE_CHECKING, Union
 
-from rdflib import BNode, Graph, Namespace, RDF, URIRef
+from rdflib import Graph, Namespace, RDF, URIRef
+from skolemizer import Skolemizer
 
 from .uri import URI
 
@@ -48,9 +49,10 @@ class Relationship:
     _had_role: URI
     _ref: URIRef
 
-    def __init__(self) -> None:
+    def __init__(self, identifier: Optional[str] = None) -> None:
         """Inits an object with default values."""
-        pass
+        if identifier:
+            self.identifier = identifier
 
     @property
     def identifier(self: Relationship) -> str:
@@ -82,7 +84,7 @@ class Relationship:
     # -
     def to_rdf(
         self: Relationship, format: str = "turtle", encoding: Optional[str] = "utf-8"
-    ) -> str:
+    ) -> Union[bytes, str]:
         """Maps the relationship to rdf.
 
         Args:
@@ -90,22 +92,22 @@ class Relationship:
             encoding: the encoding to serialize into
 
         Returns:
-            a rdf serialization as a string according to format.
+            a rdf serialization as a bytes literal according to format.
         """
         return self._to_graph().serialize(format=format, encoding=encoding)
 
     # -
     def _to_graph(self: Relationship) -> Graph:
 
+        if not getattr(self, "identifier", None):
+            self.identifier = Skolemizer.add_skolemization()
+
         # set up graph and namespaces:
         self._g = Graph()
         self._g.bind("dct", DCT)
         self._g.bind("dcat", DCAT)
 
-        if getattr(self, "identifier", None):
-            self._ref = URIRef(self.identifier)
-        else:
-            self._ref = BNode()
+        self._ref = URIRef(self.identifier)
         self._g.add((self._ref, RDF.type, DCAT.Relationship))
 
         if getattr(self, "relation", None):
